<?php

/**
 * Třída objektu uživatele
 *
 * @package    OTP
 * @author     Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Petr Jajtner © 2016
 */
class Model_User extends Zend_Db_Table_Row_Abstract {

  const EXPIRY_TIME   =  300; //  5 min.
  const LOGIN_TIMEOUT = 1200; // 20 min.

  /**
   * Název třídy modelu databáze
   *
   * @var string
   */
  protected $_tableClass = 'Model_Users';

  /**
   * Zend View
   *
   * @var Zend_View
   */
  protected $_view = null;

  /**
   * Session
   *
   * @var Zend_Session_Namespace
   */
  protected $_session = null;

  /**
   * Nově vygenerované OTP
   *
   * @var string
   */
  private $_password = '';

  /**
   * Rutiny po inicializaci objektu
   *
   * @return void
   */
  public function init() {
    $this->_view = Zend_Registry::get('Zend_View');
    $this->_session = new Zend_Session_Namespace($this->email);
    $this->_session->hasAccess = false;
  }

  /**
   * Vrátí celé jméno uživatele
   *
   * @return string
   */
  public function getFullName() {
    return trim("{$this->first_name} {$this->last_name}");
  }

  /**
   * Připraví data uživatele k použití OTP
   *
   * @return Model_User
   */
  public function prepareUse() {
    $otpSeed = Model_OTP::GeneratePassword();
    $this->_session->otp_b32 = Model_OTP::Base32Encode($otpSeed);

    $this->otp_seed = Model_OTP::UnsafeEncrypt($otpSeed, $this->email);
    $this->hotp_counter = Model_Users::OTP_TYPE_HOTP == $this->otp_type ? 0 : null;
    $this->user_password = password_hash($this->user_password, PASSWORD_BCRYPT, array('cost' => 12));

    $this->sendEmail();

    return $this;
  }

  /**
   * Ověří jednorázové heslo
   *
   * @param string $otp
   * @return boolean
   */
  public function verifyOTP($otp) {
    $otpSeed = Model_OTP::UnsafeDecrypt($this->otp_seed, $this->email);

    $result = false;
    if (Model_Users::OTP_TYPE_HOTP == $this->otp_type) {
      $counter = $this->hotp_counter + 1;
      if ($result = Model_OTP::VerifyHOTP($otp, $otpSeed, $counter)) {
        $this->hotp_counter = $counter;
      }
    }
    if (Model_Users::OTP_TYPE_TOTP == $this->otp_type) {
      $result = Model_OTP::VerifyTOTP($otp, $otpSeed);
    }

    return $result;
  }

  /**
   * Povolí přístup uživateli
   *
   * @return Model_User
   */
  public function allowAccess() {
    $this->valid_to = null;
    $this->login_at = date('Y-m-d H:i:s');
    $this->setValid()->save();
    return $this;
  }

  /**
   * Nastaví platnost
   *
   * @return Model_User
   */
  public function setValid($valid = true) {
    $this->_session->hasAccess = $valid;
    return $this;
  }

  /**
   * Otestuje, zdali má uživatel platnou session pro přístup
   *
   * @return boolean
   */
  public function hasAccess() {
    return $this->_session->hasAccess;
  }

  /**
   * Odešle email uživateli
   *
   * @return boolean | string
   */
  private function sendEmail() {
    $view = Zend_Registry::get('Zend_View');

    $sender = 'noreply@'.$_SERVER['SERVER_NAME'];
    $appName = $view->application_title;
    $message = $view->translate('LNG_USER_MODEL_EMAIL_ROW_1')."\n"
             . $view->translate('LNG_USER_MODEL_EMAIL_ROW_2')."\n"
             . $view->translate(
                 'LNG_USER_MODEL_EMAIL_ROW_3',
                 $view->translate(Model_Users::$OTP_TYPES[$this->otp_type]),
                 Model_OTP::$DIGITS,
                 $this->_session->otp_b32
               )."\n"
             . $view->translate('LNG_USER_MODEL_EMAIL_ROW_4')."\n"
             . $view->translate(
                 'LNG_USER_MODEL_EMAIL_ROW_5',
                 Model_OTP::QRCodeInBase64(
                   $this->otp_type,
                   $appName,
                   $this->_session->otp_b32,
                   $this->email,
                   $this->hotp_counter
                 )
               )."\n"
             . $view->translate('LNG_USER_MODEL_EMAIL_ROW_6')."\n"
             . $view->translate('LNG_USER_MODEL_EMAIL_ROW_7');

    try {
      $mail = new Zend_Mail('UTF-8');
      $mail->setFrom($sender, $appName);
      $mail->setReplyTo($sender, $appName);
      $mail->addTo($this->email);
      $mail->setSubject($view->translate('LNG_USER_MODEL_EMAIL_SUBJECT'));
      $mail->setBodyHtml(nl2br($message), 'UTF-8');
      $mail->send();

      return true;
    } catch (Exception $e) {
      return $e->getMessage();
    }
  }

}
